#! /usr/bin/env python
###############################################################################
#
# Copyright (C) 2005  Peeter Vois
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

import os, sys, string, StringIO, socket, time, array, re

from executions import *
import gdb_rsp

class sourceoper:
	""" Operations with source code.
	"""
	binaryfn = 'test_code.elf'	# filename
	imagefn = 'test_code.bin'	# filename
	command = ''				# command for compilation
	target = ''					# serial protocol handler
		
	def __del__ ( self ):
		self.stop()
		
	def write_code( self, code ):
		""" Writes source to temporary file.
		"""
		# take command line, the first line
		sio = StringIO.StringIO( code )
		self.command = sio.readline().split()
		sio.close
		self.command.pop(0)
		idx = self.command.index( '<src>' )
		assert idx > 0, 'command line error: <src> in wrong place'
		self.command[idx] = self.sourcefn
		idx = self.command.index( '<elf>' )
		assert idx > 0, 'command line error: <elf> in wrong place'
		self.command[idx] = self.binaryfn
		# write source
		codefd = open( self.sourcefn, 'w' )
		codefd.write( code )
		codefd.close()

	def compile( self ):
		""" Starts compilation.
		"""
		a = execute( self.command )
		hexflags = [ 'avr-objcopy', '-j', '.text', '-j', '.data', '-O', 'binary',\
			self.binaryfn, self.imagefn ]
		a.wait()
		del a
		a = execute( hexflags )
		a.wait()
		del a

	def loadtags( self ):
		""" Loads tags from created elf file
		"""
		r, w = os.pipe()
		report = os.fdopen( r )
		avrcommand = [ 'avr-nm', self.binaryfn ]
		a = execute( avrcommand, w )
		a.wait()
		del a
		os.close( w )
		# taking the tags ...
		self.tagdict = {}
		while 1:
			line = report.readline()
			if not line:
				break
			strngs = string.split( line )
			#print strngs
			self.tagdict[ strngs[2] ] = string.atoi( strngs[0], 16 )
		report.close()
		#print self.tagdict	
		
	def start( self, device, port=1212, prog='simulavr' ):
		""" Starts emulator with the device
		"""
		# starting the simulator
		command = [ prog, '-g', '-G', '-d', device, '-p', str(port) ]
		self.process = execute_tofile( command, 'sim.out', 'sim.err' )
		# starting the serial protocol handler
		tries = 5
		while (tries > 0):
			try:
				self.target = gdb_rsp.GdbRemoteSerialProtocol()
			except socket.error:
				print >> sys.stderr, 'Simulator not responding, wait a second and try again'
				tries -= 1
				time.sleep(1)
			else:
				break
		if tries == 0:
			print >> sys.stderr, 'Fatal error: simulator did not start'
			sys.exit(1)
		# loading data into
		f = open(self.imagefn)
		bin = array.array('B', f.read())
		self.target.write_mem( 0x0, len( bin ), bin )
		f.close()
	
	def stop( self ):
		""" Stops the service emulator.
		"""
		if self.process:
			#print 'deleting simulator'
			del self.process
		if self.target:
			del self.target
			
	def reset( self ):
		""" Resets target.
		"""
		self.target.cont_with_signal(signal.SIGHUP)

	
	def write_float( self, tag, value ):
		""" Wrtites value into target memory described with tag.
		"""	
		a = array.array( 'f', [ float(value) ] )
		#a.byteswap()
		b = a.tostring()
		c = array.array( 'B', b )
		self.target.write_mem( self.tagdict[ tag ], 4, c )
		
	def read_float( self, tag ):
		""" Reads value from target memory described with tag.
		"""
		a = self.target.read_mem( self.tagdict[ tag ], 4 )
		b = array.array( 'f', a.tostring() )
		#b.byteswap()
		return b[0]
		
	def set_breakpoint( self, tag ):
		""" Sets breakpoint to addres described by tag.
		"""
		self.target.break_insert( 0, self.tagdict[ tag ], 2 )
		
	def wait_breakpoint( self ):
		""" Waits until breakpoint is reached. Returns tag name of breakpoint
		"""
		a = self.target.cont()
		b = re.split( '[:;]', a )
		c = string.atoi( b[5][6:8]+b[5][4:6]+b[5][2:4]+b[5][0:2], 16 )
		print c,
		for name, value in self.tagdict.items():
			if value == c:
				break
		print name
		return name




class c_source( sourceoper ):
	""" Composes the piece of code into AVR project and compiles it.
	"""
	
	sourcefn = 'test_source.c'

	def __init__ ( self, code ):
		print code
		self.write_code( code )
		self.compile()
		self.loadtags()
		
